# Autogenerated by nbdev

d = { 'settings': { 'allowed_cell_metadata_keys': '',
                'allowed_metadata_keys': '',
                'audience': 'Developers',
                'author': 'algovera',
                'author_email': 'hello@algovera.ai',
                'black_formatting': 'False',
                'branch': 'master',
                'clean_ids': 'True',
                'company_name': 'algovera',
                'copyright': 'algovera.ai',
                'custom_sidebar': 'False',
                'description': 'A python library by Algovera to interact with IPFS and IPFS ecosystem such as the common pinning services',
                'dev_requirements': 'fastai',
                'doc_baseurl': '/ipfspy/',
                'doc_host': 'https://algoveraai.github.io',
                'doc_path': '_docs',
                'git_url': 'https://github.com/AlgoveraAI/ipfspy/tree/master',
                'host': 'github',
                'jupyter_hooks': 'True',
                'keywords': 'ipfs, web3, storage, pinata, estuary',
                'language': 'English',
                'lib_name': 'ipfspy',
                'lib_path': 'ipfspy',
                'license': 'apache2',
                'min_python': '3.6',
                'nbs_path': 'nbs',
                'readme_nb': 'index.ipynb',
                'recursive': 'False',
                'repo_name': 'ipfspy',
                'requirements': 'requests==2.27.1 fastcore pandas ipfshttpclient==0.7.0 dag-cbor fsspec ipfsspec imageio',
                'status': '2',
                'title': 'ipfspy',
                'tst_flags': 'ignoretest',
                'user': 'algovera',
                'version': '0.0.3'},
  'syms': { 'ipfspy.estuaryapi': { 'ipfspy.estuaryapi.add_car': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_car',
                                   'ipfspy.estuaryapi.add_cid': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_cid',
                                   'ipfspy.estuaryapi.add_content': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_content',
                                   'ipfspy.estuaryapi.add_content_path': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_content_path',
                                   'ipfspy.estuaryapi.add_data': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_data',
                                   'ipfspy.estuaryapi.add_key': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_key',
                                   'ipfspy.estuaryapi.add_pin': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#add_pin',
                                   'ipfspy.estuaryapi.create_coll': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#create_coll',
                                   'ipfspy.estuaryapi.est_get_viewer': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#est_get_viewer',
                                   'ipfspy.estuaryapi.get_data': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_data',
                                   'ipfspy.estuaryapi.get_deal_data': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_deal_data',
                                   'ipfspy.estuaryapi.get_deal_logs': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_deal_logs',
                                   'ipfspy.estuaryapi.get_deal_status': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_deal_status',
                                   'ipfspy.estuaryapi.get_failure_logs': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_failure_logs',
                                   'ipfspy.estuaryapi.get_miner_ask': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_miner_ask',
                                   'ipfspy.estuaryapi.get_node_stats': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_node_stats',
                                   'ipfspy.estuaryapi.get_pin': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_pin',
                                   'ipfspy.estuaryapi.get_provider_stats': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#get_provider_stats',
                                   'ipfspy.estuaryapi.list_coll_content': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_coll_content',
                                   'ipfspy.estuaryapi.list_colls': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_colls',
                                   'ipfspy.estuaryapi.list_content_path': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_content_path',
                                   'ipfspy.estuaryapi.list_data': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_data',
                                   'ipfspy.estuaryapi.list_deals': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_deals',
                                   'ipfspy.estuaryapi.list_pins': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_pins',
                                   'ipfspy.estuaryapi.list_providers': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#list_providers',
                                   'ipfspy.estuaryapi.make_deal': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#make_deal',
                                   'ipfspy.estuaryapi.remove_pin': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#remove_pin',
                                   'ipfspy.estuaryapi.replace_pin': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#replace_pin',
                                   'ipfspy.estuaryapi.view_data_cid': 'https://algoveraai.github.io/ipfspy/estuaryapi.html#view_data_cid'},
            'ipfspy.ipfshttpapi': { 'ipfspy.ipfshttpapi.DownloadDir': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#downloaddir',
                                    'ipfspy.ipfshttpapi.DownloadDir.download': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#downloaddir.download',
                                    'ipfspy.ipfshttpapi.IPFSApi': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi',
                                    'ipfspy.ipfshttpapi.IPFSApi.add_items': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.add_items',
                                    'ipfspy.ipfshttpapi.IPFSApi.block_get': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.block_get',
                                    'ipfspy.ipfshttpapi.IPFSApi.block_put': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.block_put',
                                    'ipfspy.ipfshttpapi.IPFSApi.block_rm': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.block_rm',
                                    'ipfspy.ipfshttpapi.IPFSApi.block_stat': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.block_stat',
                                    'ipfspy.ipfshttpapi.IPFSApi.cat_items': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.cat_items',
                                    'ipfspy.ipfshttpapi.IPFSApi.change_gateway_type': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.change_gateway_type',
                                    'ipfspy.ipfshttpapi.IPFSApi.dag_export': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.dag_export',
                                    'ipfspy.ipfshttpapi.IPFSApi.dag_get': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.dag_get',
                                    'ipfspy.ipfshttpapi.IPFSApi.dag_stat': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.dag_stat',
                                    'ipfspy.ipfshttpapi.IPFSApi.get_items': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.get_items',
                                    'ipfspy.ipfshttpapi.IPFSApi.ls_items': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.ls_items',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_chcid': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_chcid',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_cp': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_cp',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_flush': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_flush',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_ls': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_ls',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_mkdir': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_mkdir',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_mv': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_mv',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_read': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_read',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_rm': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_rm',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_stat': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_stat',
                                    'ipfspy.ipfshttpapi.IPFSApi.mfs_write': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.mfs_write',
                                    'ipfspy.ipfshttpapi.IPFSApi.pin_add': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.pin_add',
                                    'ipfspy.ipfshttpapi.IPFSApi.pin_ls': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.pin_ls',
                                    'ipfspy.ipfshttpapi.IPFSApi.pin_rm': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.pin_rm',
                                    'ipfspy.ipfshttpapi.IPFSApi.rpin_add': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.rpin_add',
                                    'ipfspy.ipfshttpapi.IPFSApi.rpin_ls': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.rpin_ls',
                                    'ipfspy.ipfshttpapi.IPFSApi.rpin_rm': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.rpin_rm',
                                    'ipfspy.ipfshttpapi.IPFSApi.rspin_add': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.rspin_add',
                                    'ipfspy.ipfshttpapi.IPFSApi.rspin_ls': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.rspin_ls',
                                    'ipfspy.ipfshttpapi.IPFSApi.rspin_rm': 'https://algoveraai.github.io/ipfspy/ipfshttpapi.html#ipfsapi.rspin_rm'},
            'ipfspy.pinataapi': { 'ipfspy.pinataapi.edit_metadata': 'https://algoveraai.github.io/ipfspy/pinataapi.html#edit_metadata',
                                  'ipfspy.pinataapi.generate_apikey': 'https://algoveraai.github.io/ipfspy/pinataapi.html#generate_apikey',
                                  'ipfspy.pinataapi.get_datausage': 'https://algoveraai.github.io/ipfspy/pinataapi.html#get_datausage',
                                  'ipfspy.pinataapi.get_pinned_files': 'https://algoveraai.github.io/ipfspy/pinataapi.html#get_pinned_files',
                                  'ipfspy.pinataapi.get_pinned_jobs': 'https://algoveraai.github.io/ipfspy/pinataapi.html#get_pinned_jobs',
                                  'ipfspy.pinataapi.list_apikeys': 'https://algoveraai.github.io/ipfspy/pinataapi.html#list_apikeys',
                                  'ipfspy.pinataapi.pin': 'https://algoveraai.github.io/ipfspy/pinataapi.html#pin',
                                  'ipfspy.pinataapi.revoke_apikey': 'https://algoveraai.github.io/ipfspy/pinataapi.html#revoke_apikey',
                                  'ipfspy.pinataapi.unpin': 'https://algoveraai.github.io/ipfspy/pinataapi.html#unpin',
                                  'ipfspy.pinataapi.upload_file': 'https://algoveraai.github.io/ipfspy/pinataapi.html#upload_file',
                                  'ipfspy.pinataapi.upload_jsonfile': 'https://algoveraai.github.io/ipfspy/pinataapi.html#upload_jsonfile'},
            'ipfspy.utils': { 'ipfspy.utils.GATEWAY_MAP': 'https://algoveraai.github.io/ipfspy/utils.html#gateway_map',
                              'ipfspy.utils.IPFSGateway': 'https://algoveraai.github.io/ipfspy/utils.html#ipfsgateway',
                              'ipfspy.utils.IPFSGateway.apipost': 'https://algoveraai.github.io/ipfspy/utils.html#ipfsgateway.apipost',
                              'ipfspy.utils.IPFSGateway.get': 'https://algoveraai.github.io/ipfspy/utils.html#ipfsgateway.get',
                              'ipfspy.utils.IPFSGateway.get_state': 'https://algoveraai.github.io/ipfspy/utils.html#ipfsgateway.get_state',
                              'ipfspy.utils.IPFSGateway.head': 'https://algoveraai.github.io/ipfspy/utils.html#ipfsgateway.head',
                              'ipfspy.utils.parse_error_message': 'https://algoveraai.github.io/ipfspy/utils.html#parse_error_message',
                              'ipfspy.utils.parse_response': 'https://algoveraai.github.io/ipfspy/utils.html#parse_response'}}}


# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_estuaryapi.ipynb.

# %% auto 0
__all__ = ['est_get_viewer', 'list_pins', 'add_pin', 'get_pin', 'replace_pin', 'remove_pin', 'create_coll', 'add_content',
           'list_colls', 'list_coll_content', 'list_content_path', 'add_content_path', 'add_key', 'add_data', 'add_cid',
           'add_car', 'make_deal', 'view_data_cid', 'list_data', 'list_deals', 'get_deal_status', 'get_node_stats',
           'get_deal_data', 'get_miner_ask', 'get_failure_logs', 'get_deal_logs', 'get_provider_stats',
           'list_providers', 'get_data']

# %% ../nbs/02_estuaryapi.ipynb 3
import requests
from .utils import parse_response

# %% ../nbs/02_estuaryapi.ipynb 4
def est_get_viewer(
    api_key: str # Your Estuary API key
):
    "View your Estuary account details"
    headers = {
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.get('https://api.estuary.tech/viewer', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 6
# list pins
def list_pins(
    api_key: str # Your Estuary API key
):
    "List all your pins"

    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.get('https://api.estuary.tech/pinning/pins', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 7
# add pin
def add_pin(
    api_key: str, # Your Estuary API key
    file_name: str, # File name to pin
    cid: str # CID to attach
):
    "Add a new pin object for the current access token."

    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    json_data = {
        'name': name,
        'cid': cid,
    }

    response = requests.post('https://api.estuary.tech/pinning/pins', headers=headers, json=json_data)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 8
# get pin by ID
def get_pin(
    api_key: str, # Your Estuary API key
    pin_id: str # Unique pin ID
):
    "Get a pinned object by ID"
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.get(f'https://api.estuary.tech/pinning/pins/{pin_id}', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 9
# replace pin by ID
def replace_pin(
    api_key: str, # Your Estuary API key
    pin_id: str # Unique pin ID
):
    "Replace a pinned object by ID"
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.post(f'https://api.estuary.tech/pinning/pins/{pin_id}', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 10
# remove pin by ID
def remove_pin(
    api_key: str, # Your Estuary API key
    pin_id: str # Unique pin ID
):
    "Remove a pinned object by ID"
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.delete(f'https://api.estuary.tech/pinning/pins/{pin_id}', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 12
# create new collection
def create_coll(
    api_key: str, # Your Estuary API key
    name: str, # Collection name
    description: str # Collection description
):
    "Create new collection"
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    json_data = {
        'name': name,
        'description': description,
    }

    response = requests.post('https://api.estuary.tech/collections/create', headers=headers, json=json_data)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 13
# add content
def add_content(
    api_key: str, # Your Estuary API key
    collection_id: str, # Collection ID
    data: list, # List of paths to data to be added
    cids: list # List of respective CIDs
    
):
    "Add data to Collection"
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    json_data = {
        'contents': data,
        'cids': cids,
        'collection': collection_id,
    }

    response = requests.post('https://api.estuary.tech/collections/add-content', headers=headers, json=json_data)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 14
# list collections
def list_colls(
    api_key: str # Your Estuary API key
):
    "List your collections"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.get('https://api.estuary.tech/collections/list', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 15
# list collection content
def list_coll_content(
    api_key: str, # Your Estuary API key
    collection_id: str # Collection ID
):
    "List contents of a collection from ID"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    }

    response = requests.get(f'https://api.estuary.tech/collections/content/{collection_id}', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 16
# FS list content of a path
def list_content_path(
    api_key: str, # Your Estuary API key
    collection_id: str, # Collection ID
    path: str # Path in collection to list files from
):
    "List content of a path in collection"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    }

    params = {
        'col': collection_id,
    }

    response = requests.get(f'https://api.estuary.tech/collections/fs/list?col=UUID&dir={path}', params=params, headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 17
# FS add content to path
def add_content_path(
    api_key: str, # Your Estuary API key
    collection_id: str, # Collection ID
    path: str # Path in collection to add files to
):
    "Add content to a specific file system path in an IPFS collection"
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    params = {
        'col': collection_id,
    }

    response = requests.post(f'https://api.estuary.tech/collections/fs/add?col=UUID&content=LOCAL_ID&path={path}', params=params, headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 19
# add client safe upload key
def add_key(
    api_key, # Your Estuary API key
    expiry='24h' # Expiry of upload key
):
    "Add client safe upload key"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    'Accept': 'application/json',
    }

    params = {
        'perms': 'upload',
        'expiry': expiry,
    }

    response = requests.post('https://api.estuary.tech/user/api-keys', params=params, headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 20
def add_data(
    api_key: str, # Your Estuary API key
    path_to_file: str # Path to file you want to upload
):
    "Upload file to Estuary"

    headers = {
    'Authorization': f'Bearer {api_key}',
    'Accept': 'application/json',
    }

    files = {
        'data': open(path_to_file, 'rb'),
    }

    response = requests.post('https://api.estuary.tech/content/add', headers=headers, files=files)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 21
# add CID
def add_cid(
    api_key: str, # Your Estuary API key
    file_name: str, # File name to add to CID
    cid: str # CID for file
):
    "Use an existing IPFS CID to make storage deals."
    
    headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
    }

    json_data = {
        'name': file_name,
        'root': cid,
    }

    response = requests.post('https://api.estuary.tech/content/add-ipfs', headers=headers, json=json_data)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 22
# add CAR
def add_car(
    api_key: str, # Your Estuary API key
    path_to_file: str # Path to file to store
):
    "Write a Content-Addressable Archive (CAR) file, and make storage deals for its contents."
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    'Accept': 'application/json',
    }

    with open(path_to_file, 'rb') as f:
        data = f.read()

    response = requests.post('https://api.estuary.tech/content/add-car', headers=headers, data=data)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 23
# make deal with specific provider
def make_deal(
    api_key: str, # Your Estuary API key
    content_id: str, # Content ID on Estuary
    provider_id: str, # Provider ID
):
    "Make a deal with a storage provider and a file you have already uploaded to Estuary"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    }

    json_data = {
        'content': content_id,
    }

    response = requests.post(f'https://api.estuary.tech/deals/make/{provider_id}', headers=headers, json=json_data)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 24
# data by CID
def view_data_cid(
    api_key: str, # Your Estuary API key
    cid: str # CID
):
    "View CID information"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    }
    
    response = requests.get(f'https://api.estuary.tech/content/by-cid/{cid}', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 25
# list data
def list_data(
    api_key: str # Your Estuary API key
):
    headers = {
    'Authorization': f'Bearer {api_key}',
    }
    
    response = requests.get('https://api.estuary.tech/content/stats', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 26
# list deals
def list_deals(
    api_key: str # Your Estuary API key
):
    headers = {
    'Authorization': f'Bearer {api_key}',
    }
    
    response = requests.get('https://api.estuary.tech/content/deals', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 27
# get deal status by id
def get_deal_status(
    api_key: str, # Your Estuary API key
    deal_id: str # Deal ID
):
    "Get deal status by id"
    
    headers = {
    'Authorization': f'Bearer {api_key}',
    }
    
    response = requests.get(f'https://api.estuary.tech/content/status/{deal_id}', headers=headers)
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 28
# get Estuary node stats
def get_node_stats():
    "Get Estuary node stats"

    response = requests.get('https://api.estuary.tech/public/stats')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 29
# get on chain deal data
def get_deal_data():
    "Get on-chain deal data"

    response = requests.get('https://api.estuary.tech/public/metrics/deals-on-chain')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 30
# get miner query ask
def get_miner_ask(
    miner_id: str # Miner ID
):
    "Get the query ask and verified ask for any miner"
    
    response = requests.get(f'https://api.estuary.tech/public/miners/storage/query/{miner_id}')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 31
# get failure logs by provider
def get_failure_logs(
    miner_id: str # Miner ID
):
    "Get all of the failure logs for a specific miner"
    
    response = requests.get(f'https://api.estuary.tech/public/miners/failures/{miner_id}')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 32
# get deal logs by provider
def get_deal_logs(
    provider_id: str # Provider ID
):
    "Get deal logs by provider"
    
    response = requests.get(f'https://api.estuary.tech/public/miners/deals/{provider_id}')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 33
# get provider stats
def get_provider_stats(
    provider_id: str # Provider ID
):
    "Get provider stats"
    
    response = requests.get(f'https://api.estuary.tech/public/miners/stats/{provider_id}')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 34
# list providers
def list_providers():
    "List Estuary providers"
    
    response = requests.get('https://api.estuary.tech/public/miners')
    return response, parse_response(response)

# %% ../nbs/02_estuaryapi.ipynb 36
def get_data(
    cid: str, # Data CID
    path_name: str # Path and filename to store the file at
):
    "Download data from Estuary CID"
    
    url = f'https://dweb.link/ipfs/{cid}'
    response = requests.get(url, allow_redirects=True)  # to get content
    with open(path_name, 'wb') as f:
        f.write(response.content)
    return response, parse_response(response)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_ipfshttpapi.ipynb.

# %% auto 0
__all__ = ['IPFSApi', 'DownloadDir']

# %% ../nbs/01_ipfshttpapi.ipynb 4
from typing import Union, List

import requests
import json
from fastcore.all import *
import pandas as pd
import dag_cbor
from requests.exceptions import HTTPError
import time

from .utils import parse_error_message, parse_response, IPFSGateway, GATEWAY_MAP

from ipfshttpclient.multipart import stream_files, stream_directory

# %% ../nbs/01_ipfshttpapi.ipynb 5
class IPFSApi():
    def __init__(self,
        gateway_type='local', # Gateway to use - `public`, `infura`, `local` 
        timeout=10, # 
        **kwargs
    ):
        
        self.gateway = None
        self._gateways = None

        self.change_gateway_type = gateway_type
        self.timeout = timeout
        self.full_structure = None
    
    @property
    def change_gateway_type(self):
        return self.gateway_type

    @change_gateway_type.setter    
    def change_gateway_type(self, 
        value, 
        ):
        self.gateway_type = value
        self._gateways = [IPFSGateway(g) for g in GATEWAY_MAP[self.gateway_type]] 
        print(f"Changed to {self.gateway_type} node")
    
    def _find_gateway(self):
        backoff_list = []
        for gw in self._gateways:
            state, wait_time = gw.get_state()
            if state == "online":
                return gw, 0
            if state == "backoff":
                backoff_list.append((wait_time, gw))
        if len(backoff_list) > 0:
            return sorted(backoff_list)[0][::-1]
        else:
            raise RuntimeError("no working gateways could be found")

    def _run_on_any_gateway(self, f):
        timeout = time.monotonic() + self.timeout
        while time.monotonic() <= timeout:
            gw, wait_time = self._find_gateway()
            if wait_time > 0:
                time.sleep(wait_time)
            res = f(gw)
            if res is not None:
                break
        return res
        
    def _gw_get(self, path):
        return self._run_on_any_gateway(lambda gw: gw.get(path))

    def _gw_head(self, path, headers=None):
        return self._run_on_any_gateway(lambda gw: gw.head(path, headers))

    def _gw_apipost(self, call, **kwargs): 
        return self._run_on_any_gateway(lambda gw: gw.apipost(call, **kwargs))
    
    def add_items(self,
        filepath:Union[str, List[str]], # Path to the file/directory to be added to IPFS
        directory:bool=False, # Is filepath a directory
        wrap_with_directory:bool=False, # True if path is a directory
        chunker:str='size-262144', # Chunking algorithm, size-[bytes], rabin-[min]-[avg]-[max] or buzhash
        pin:bool=True, # Pin this object when adding
        hash_:str='sha2-256', # Hash function to use. Implies CIDv1 if not sha2-256
        progress:str='true', # Stream progress data
        silent:str='false', # Write no output
        cid_version:int=0, # CID version
        **kwargs,
        ):
        "add file/directory to ipfs"

        params = {}
        params['wrap-with-directory'] = 'true' if wrap_with_directory else 'false'
        params['chunker'] = chunker
        params['pin'] = 'true' if pin else 'false'
        params['hash'] = hash_
        params['progress'] = progress
        params['silent'] = silent
        params['cid-version'] = cid_version
        params.update(kwargs)
        
        chunk_size = int(chunker.split('-')[1])

        if not directory:
            data, headers = stream_files(filepath, chunk_size=chunk_size)

        else:
            data, headers = stream_directory(filepath, chunk_size=chunk_size, recursive=True)

        response = self._gw_apipost('add', 
                                    params=params, 
                                    data=data,
                                    headers=headers)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def ls_items(self,
        cid:str, # The path to the IPFS object(s) to list links from
        resolve_type:bool=True, # Resolve linked objects to find out their types
        size:bool=True, # Resolve linked objects to find out their file size
        **kwargs,
    ):
        'List directory contents for Unix filesystem objects'

        params = {}
        params['arg'] = cid
        params['resolve-type'] = 'true' if resolve_type else 'false'
        params['size'] = 'true' if size else 'false'
        params.update(kwargs)

        response = self._gw_apipost('ls', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def get_items(self,
        cid:str, # The path to the IPFS object(s) to be outputted
        output:str='', # The path where the output should be stored
        **kwargs
    ):
        'Download IPFS objects'

        params = {}
        params['arg'] = cid
        params['output'] = output
        params.update(kwargs)

        response = self._gw_apipost('get', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def cat_items(self,
        cid:str, # The path to the IPFS object(s) to be output
        **kwargs
    ):
        'Show IPFS object data'

        params = {}
        params['arg'] = cid
        params.update(kwargs)

        response = self._gw_apipost('cat', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
    
    def dag_export(self,
        cid:str, #The path to the IPFS DAG node
        **kwargs,
    ):
        'Streams the selected DAG as a .car stream on stdout.'

        params = {}
        params['arg'] = cid
        params.update(kwargs)

        response = self._gw_apipost('dag/export', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def dag_get(self,
        cid:str, #The path to the IPFS DAG node
        output_codec:str='dag-json',
    ):
        'Get a DAG node from IPFS.'

        params = {}
        params['arg'] = cid
        params['output-codec'] = output_codec

        response = self._gw_apipost('dag/get', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def dag_stat(self,
        cid:str, #The path to the IPFS DAG node 
        **kwargs, 
    ):
        'Gets stats for a DAG.'

        params = {}
        params['arg'] = cid
        params.update(kwargs)

        response = self._gw_apipost('dag/stat', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
    
    def pin_add(self,
        cid:str, # Path to IPFS object(s) to be pinned
        recursive:str='true', # Recursively pin the object linked to by the specified object(s)
    ):
        'Pin objects to local storage.'

        params = {}
        params['arg'] = cid
        params['recursive'] = recursive

        response = self._gw_apipost('pin/add', params=params)
        
        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def pin_ls(self,
        type_:str='all', # The type of pinned keys to list. Can be "direct", "indirect", "recursive", or "all"
        **kwargs,
    ):
        'List objects pinned to local storage.'    

        params = {}
        params['type'] = type_
        params.update(kwargs)

        response = self._gw_apipost('pin/ls', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def pin_rm(self,
        cid:str, # Path to object(s) to be unpinned
        recursive:str='true', #  Recursively unpin the object linked to by the specified object(s)
        **kwargs,
    ):
        'List objects pinned to local storage.'    

        params = {}
        params['arg'] = cid
        params['recursive'] = recursive
        params.update(kwargs)

        response = self._gw_apipost('pin/rm', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def rspin_add(self,
        service_name:str, # Name of the remote pinning service to use
        service_edpt:str, # Service endpoint
        service_key:str, # Service key
    ):
        'Pin object to remote pinning service.'

        params = {}
        params['arg'] = [service_name, service_edpt, service_key]

        response = self._gw_apipost('pin/remote/service/add', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def rspin_ls(self,
        **kwargs
    ):
        'List remote pinning services.'

        params = {}
        params.update(kwargs)

        response = self._gw_apipost('pin/remote/service/ls', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))


    def rspin_rm(self,
        service_name:str, # Name of pinning service to remove
    ):
        'Remove remote pinning service.'

        params = {}
        params['arg'] = service_name

        response = self._gw_apipost('pin/remote/service/rm', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
    
    def rpin_add(self,
        cid:str, #  Path to IPFS object(s) to be pinned
        service:str, # Name of the remote pinning service to use
        background:str='false', # Add to the queue on the remote service and return immediately (does not wait for pinned status)
        **kwargs,
    ):
        'Pin object to remote pinning service.'

        params = {}
        params['arg'] = cid
        params['service'] = service
        params['background'] = background
        params.update(kwargs)

        response = self._gw_apipost('pin/remote/add', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))


    def rpin_ls(self,
        service:str, # Name of the remote pinning service to use
        **kwargs, 
    ):
        'List objects pinned to remote pinning service.'

        params = {}
        params['service'] = service
        params.update(kwargs)

        response = self._gw_apipost('pin/remote/ls', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def rpin_rm(self,
        service:str, # Name of the remote pinning service to use
        **kwargs, 
    ):
        'Remove pins from remote pinning service.'    

        params = {}
        params['service'] = service
        params.update(kwargs)

        response = self._gw_apipost('pin/remote/rm', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
            
    def block_get(self,
        cid:str, # The base58 multihash of an existing block to get

    ):
        'Get a raw IPFS block.'

        params = {}
        params['arg'] = cid

        response = self._gw_apipost('block/get', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def block_put(self,
        filepath:str, # Path to file
        mhtype:str='sha2-256', # multihash hash function.
        mhlen:int=-1, # Multihash hash length
        pin:str=False, #  pin added blocks recursively
        **kwargs,
    ):
        'Store input as an IPFS block.'

        params = {}
        params['mhtype'] = mhtype
        params['mhlen'] = mhlen
        params['pin'] = 'true' if pin else 'false'
        params.update(kwargs)

        response = self._gw_apipost('block/put', params=params, files={'files':open(filepath, 'rb')})

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
            
    def block_rm(self,
        cid:str, # Bash58 encoded multihash of block(s) to remove
        force:str='false', # Ignore nonexistent blocks.
        quiet:str='false', # Write minimal output.
    ):
        'Remove IPFS block(s).'

        params = {}
        params['arg'] = cid
        params['force'] = force
        params['quiet'] = quiet

        response = self._gw_apipost('block/rm', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def block_stat(self,
        cid:str, # Bash58 encoded multihash of block(s) to remove

    ):
        'Print information of a raw IPFS block.'

        params = {}
        params['arg'] = cid

        response = self._gw_apipost('block/stat', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
            
    def mfs_chcid(self,
        path:str='/', # Path to change
        cid_version:int=0, # Cid version to use
        **kwargs,
    ):
        'Change the CID version or hash function of the root node of a given path.'

        params = {}
        params['arg'] = path
        params['cid-version'] = cid_version
        params.update(kwargs)

        response = self._gw_apipost("files/chcid", params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_cp(self,
        source_path:str, # Source IPFS or MFS path to copy
        dest_path:str, # Destination within MFS
        **kwargs
    ):
        'Add references to IPFS files and directories in MFS (or copy within MFS).'

        params = {}
        params['arg'] = [source_path, dest_path]
        params.update(kwargs)

        response = self._gw_apipost('files/cp', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_flush(self,
        path:str='/', # Path to flush
    ):
        "Flush a given path's data to disk"

        params['arg'] = path

        response = self._gw_apipost('files/flush',params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_ls(self, 
        path:str='/', # Path to show listing for 
        **kwargs
    ):
        "List directories in the local mutable namespace."

        params = {}
        params['arg'] = path
        params.update(kwargs)

        response = self._gw_apipost('files/ls', params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_mkdir(self,
        path:str, # Path to dir to make
        **kwargs,
    ):
        "Make directories."

        params = {}
        params['arg'] = path
        params.update(kwargs)

        response = self._gw_apipost("files/mkdir", params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_mv(self,
        source_path:str, # Source file to move
        dest_path:str,  # Destination path for file to be moved to
    ):
        "Move files."

        params = {}
        params['arg'] = [source_path, dest_path]

        response = self._gw_apipost("files/mv", params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_read(self,
        path, # Path to file to be read
        **kwargs,
    ):
        "Read a file in a given MFS."

        params = {}
        params['arg'] = path
        params.update(kwargs)

        response = self._gw_apipost("files/read",params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))
            
    def mfs_rm(self,
        path, # File to remove
        **kwargs,
    ):
        "Read a file in a given MFS."

        params = {}
        params['arg'] = path
        params.update(kwargs)

        response = self._gw_apipost("files/rm", params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_stat(self,
        path, # Path to node to stat
        **kwargs,
    ):
        "Display file status."    

        params = {}
        params['arg'] = path
        params.update(kwargs)

        response = self._gw_apipost("files/stat", params=params)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

    def mfs_write(self,
        path, # Path to write to
        filepath, # File to add
        create=True, # Create the file if it does not exist
        **kwargs,
    ):
        "Display file status."    

        files = {
            'file': open(filepath, 'rb')
        }

        params = {}
        params['arg'] = path
        params['create'] = 'true' if create else 'false'
        params.update(kwargs)

        response = self._gw_apipost("files/write", params=params, files=files)

        if response.status_code == 200:
            return response, parse_response(response)

        else:
            raise HTTPError (parse_error_message(response))

# %% ../nbs/01_ipfshttpapi.ipynb 52
class DownloadDir:
    'Download a IPFS directory to your local disk'
    def __init__(self,
        gateway_type:str, # Gateway to use - works on local and public 
        root_cid:str, # Root CID of the directory
        output_fol:str, # Path to save in your local disk
    ):
        
        self.api = IPFSApi(gateway_type=gateway_type)
        self.root = root_cid
        self.output = output_fol
        self.full_structure = None
    
    def _get_links(self,
        cid, 
        fol
    ):
        root_struct = {}
        struct = {}

        _, obj = self.api.ls_items(cid)
        
        for link in obj[0]['Objects'][0]['Links']:
            name = f'{fol}/{link["Name"]}'
            hash_ = str(link['Hash'])
            type_ = link['Type']

            if type_ != 2:
                details = self._get_links(hash_, name)

            else:
                details = {'Hash': hash_, 'type': type_}

            struct[name] = details

        root_struct[fol] = struct

        return root_struct
    
    
    def _save_links(self,
        links
    ):
        for k, v in links.items():
            if len(k.split('.')) < 2:
                if not os.path.exists(k): os.mkdir(k)
                self._save_links(v)

            else:
                res, data = self.api.cat_items(links[k]['Hash'])

                with open(k, 'wb') as f:
                    f.write(res.content)
                    
    def download(self
    ):
        self.full_structure = self._get_links(self.root, self.output)
        self._save_links(self.full_structure)
        



# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"parse_response": "00_utils.ipynb",
         "parse_error_message": "00_utils.ipynb",
         "GATEWAY_MAP": "00_utils.ipynb",
         "IPFSGateway": "00_utils.ipynb",
         "IPFSApi": "01_ipfshttpapi.ipynb",
         "DownloadDir": "01_ipfshttpapi.ipynb",
         "est_get_viewer": "02_estuaryapi.ipynb",
         "list_pins": "02_estuaryapi.ipynb",
         "add_pin": "02_estuaryapi.ipynb",
         "get_pin": "02_estuaryapi.ipynb",
         "replace_pin": "02_estuaryapi.ipynb",
         "remove_pin": "02_estuaryapi.ipynb",
         "create_coll": "02_estuaryapi.ipynb",
         "add_content": "02_estuaryapi.ipynb",
         "list_colls": "02_estuaryapi.ipynb",
         "list_coll_content": "02_estuaryapi.ipynb",
         "list_content_path": "02_estuaryapi.ipynb",
         "add_content_path": "02_estuaryapi.ipynb",
         "add_key": "02_estuaryapi.ipynb",
         "add_data": "02_estuaryapi.ipynb",
         "add_cid": "02_estuaryapi.ipynb",
         "add_car": "02_estuaryapi.ipynb",
         "make_deal": "02_estuaryapi.ipynb",
         "view_data_cid": "02_estuaryapi.ipynb",
         "list_data": "02_estuaryapi.ipynb",
         "list_deals": "02_estuaryapi.ipynb",
         "get_deal_status": "02_estuaryapi.ipynb",
         "get_node_stats": "02_estuaryapi.ipynb",
         "get_deal_data": "02_estuaryapi.ipynb",
         "get_miner_ask": "02_estuaryapi.ipynb",
         "get_failure_logs": "02_estuaryapi.ipynb",
         "get_deal_logs": "02_estuaryapi.ipynb",
         "get_provider_stats": "02_estuaryapi.ipynb",
         "list_providers": "02_estuaryapi.ipynb",
         "get_data": "02_estuaryapi.ipynb",
         "generate_apikey": "03_pinataapi.ipynb",
         "list_apikeys": "03_pinataapi.ipynb",
         "revoke_apikey": "03_pinataapi.ipynb",
         "upload_file": "03_pinataapi.ipynb",
         "upload_jsonfile": "03_pinataapi.ipynb",
         "pin": "03_pinataapi.ipynb",
         "unpin": "03_pinataapi.ipynb",
         "edit_metadata": "03_pinataapi.ipynb",
         "get_pinned_jobs": "03_pinataapi.ipynb",
         "get_pinned_files": "03_pinataapi.ipynb",
         "get_datausage": "03_pinataapi.ipynb"}

modules = ["utils.py",
           "ipfshttpapi.py",
           "estuaryapi.py",
           "pinataapi.py"]

doc_url = "algoveraai.github.io/ipfspy/"

git_url = "https://github.com/AlgoveraAI/ipfspy/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_utils.ipynb.

# %% auto 0
__all__ = ['GATEWAY_MAP', 'parse_response', 'parse_error_message', 'IPFSGateway']

# %% ../nbs/00_utils.ipynb 3
import requests
import json
from fastcore.all import *
import pandas as pd
import dag_cbor
from typing import Union
import time

# %% ../nbs/00_utils.ipynb 5
def parse_response(
    response, # Response object
):
    "Parse response object into JSON"
    
    if response.text.split('\n')[-1] == "":
        try:
            return [json.loads(each) for each in response.text.split('\n')[:-1]]
        
        except:
            pass

    try:
        return response.json()

    except:
        return response.text
    
    
def parse_error_message(
    response, # Response object from requests
):
    'Parse error message for raising exceptions'
 
    sc = response.status_code
    
    try:
        message = response.json()['Message']
        
    except:
        message = response.text
    
    return f"Response Status Code: {sc}; Error Message: {message}"

# %% ../nbs/00_utils.ipynb 6
GATEWAY_MAP = {
    'local': [f"http://127.0.0.1:5001"],
    'infura': ['https://ipfs.infura.io:5001'],
    'public': ["https://ipfs.io",
               "https://gateway.pinata.cloud",
               "https://cloudflare-ipfs.com",
               "https://dweb.link"]
}

# %% ../nbs/00_utils.ipynb 7
class IPFSGateway:
    def __init__(self, url):
        self.url = url
        self.state = "unknown"
        self.min_backoff = 1e-9
        self.max_backoff = 5
        self.backoff_time = 0
        self.next_request_time = time.monotonic()
        self.session = requests.Session()
        adapter = requests.adapters.HTTPAdapter(pool_connections=100, pool_maxsize=100)
        self.session.mount('http://', adapter)
        self.session.mount('https://', adapter)

    def get(self, path):
        try:
            res = self.session.get(self.url + "/ipfs/" + path)
        except requests.ConnectionError as e:
            self._backoff()
            return None
        # this is from https://blog.petrzemek.net/2018/04/22/on-incomplete-http-reads-and-the-requests-library-in-python/
        expected_length = res.headers.get('Content-Length')
        if expected_length is not None:
            actual_length = res.raw.tell()
            expected_length = int(expected_length)
            if actual_length < expected_length:
                # if less than the expected amount of data is delivered, just backoff which will will eiter trigger a
                # retry on the same server or will fall back to another server later on.
                self._backoff()
                return None

        if res.status_code == 429:  # too many requests
            self._backoff()
            return None
        elif res.status_code == 200:
            self._speedup()
        res.raise_for_status()
        return res.content

    def head(self, path, headers=None):
        try:
            res = self.session.get(self.url + "/ipfs/" + path)
        except requests.ConnectionError as e:
            self._backoff()
            return None
        if res.status_code == 429:  # too many requests
            self._backoff()
            return None
        elif res.status_code == 200:
            self._speedup()
        res.raise_for_status()
        return res.headers

    def apipost(self, call, **kwargs):
        try:
            data = kwargs.pop('data') if 'data'  in kwargs else {}
            headers = kwargs.pop('headers') if 'headers' in kwargs else {}
            files = kwargs.pop('files') if 'files' in kwargs else {}
            params = kwargs['params'] if 'params' in kwargs else kwargs

            res = self.session.post(self.url + "/api/v0/" + call, 
                                    params=params, 
                                    data=data, 
                                    headers=headers,
                                    files=files)
        
        except requests.ConnectionError:
            self._backoff()
            return None
        
        if res.status_code == 429:  # too many requests
            self._backoff()
            return None
        
        elif res.status_code == 200:
            self._speedup()
        
        return res

    def _schedule_next(self):
        self.next_request_time = time.monotonic() + self.backoff_time

    def _backoff(self):
        self.backoff_time = min(max(self.min_backoff, self.backoff_time) * 2,
                                self.max_backoff)
        self._schedule_next()

    def _speedup(self):
        self.backoff_time = max(self.min_backoff, self.backoff_time * 0.9)
        self._schedule_next()

    def _init_state(self):
        try:
            res = self.session.post(self.url + "/api/v0/version")
            if res.ok:
                self.state = "online"
            else:
                self.state = "offline"
        except requests.ConnectionError:
            self.state = "offline"

    def get_state(self):
        if self.state == "unknown":
            self._init_state()
        now = time.monotonic()
        if self.next_request_time > now:
            return ("backoff", self.next_request_time - now)
        else:
            return (self.state, None)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_pinataapi.ipynb.

# %% auto 0
__all__ = ['generate_apikey', 'list_apikeys', 'revoke_apikey', 'upload_file', 'upload_jsonfile', 'pin', 'unpin', 'edit_metadata',
           'get_pinned_jobs', 'get_pinned_files', 'get_datausage']

# %% ../nbs/03_pinataapi.ipynb 4
import requests
import json

# %% ../nbs/03_pinataapi.ipynb 6
def generate_apikey(cred:str,#JWT
                    key_name:str, #Key name
                    pinlist:bool=False,#list pins
                    userPinnedDataTotal:bool=False, #total data stored
                    hashMetadata:bool=True, #metadata
                    hashPinPolicy:bool=False, #policy
                    pinByHash:bool=True, #pin cid
                    pinFileToIPFS:bool=True,#upload file to IPFS
                    pinJSONToIPFS:bool=True,#upload json to IPFS
                    pinJobs:bool=True,#see pin jobs
                    unpin:bool=True,#unpin ipfs cid
                    userPinPolicy:bool=True #establish pin policy
                   
):

    url = "https://api.pinata.cloud/users/generateApiKey"

    payload = json.dumps({
      "keyName": key_name,
      "permissions": {
        "endpoints": {
          "data": {
            "pinList": pinlist,
            "userPinnedDataTotal": userPinnedDataTotal
          },
          "pinning": {
            "hashMetadata": hashMetadata,
            "hashPinPolicy": hashPinPolicy,
            "pinByHash": pinByHash,
            "pinFileToIPFS": pinFileToIPFS,
            "pinJSONToIPFS": pinJSONToIPFS,
            "pinJobs": pinJobs,
            "unpin": unpin,
            "userPinPolicy": userPinPolicy
          }
        }
      }
    })
    headers = {
      'Authorization': f'Bearer {cred}',
      'Content-Type': 'application/json'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    return response


# %% ../nbs/03_pinataapi.ipynb 9
def list_apikeys(cred:str
):

    url = "https://api.pinata.cloud/users/apiKeys"

    payload={}
    headers = {
      'Authorization': f'Bearer {cred}'
    }

    response = requests.request("GET", url, headers=headers, data=payload)
    
    return response


# %% ../nbs/03_pinataapi.ipynb 12
def revoke_apikey(cred:str,
                  revoke_apikey:str
):
    url = "https://api.pinata.cloud/users/revokeApiKey"

    payload = json.dumps({
      "apiKey": revoke_apikey
    })
    headers = {
      'Authorization': f'Bearer {cred}',
      'Content-Type': 'application/json'
    }

    response = requests.request("PUT", url, headers=headers, data=payload)

    return response


# %% ../nbs/03_pinataapi.ipynb 15
def upload_file(cred:str,#JWT key
                name:str, #filename
                fpaths:list, #filepaths
                metadata:dict, #metadata
                cid_version:str="1", #IPFS cid
                directory:bool=False #upload directory
):

    pinataMetadata = dict({"name":name,"keyvalues":{}})
    pinataMetadata["name"] = name
    pinataMetadata["keyvalues"].update(metadata)
    
    pinataOptions = dict({"cidVersion":cid_version,"directory":directory})
    

    url = "https://api.pinata.cloud/pinning/pinFileToIPFS"

    payload={"pinataOptions":json.dumps(pinataOptions),"pinataMetadata":json.dumps(pinataMetadata)}
    
    if directory:
        print("feature is not ready yet")
    
    files=[('file',(name,open(fpaths,'rb'),'application/octet-stream'))]
    
    headers = {
      'Authorization': f'Bearer {cred}'
    }

    response = requests.request("POST", url, headers=headers, data=payload, files=files)

    return response

# %% ../nbs/03_pinataapi.ipynb 18
def upload_jsonfile(cred:str,#JWT key
                name:str, #filename
                fpaths:list, #filepaths
                metadata:dict, #metadata
                cid_version:str, #IPFS cid
                directory:bool=False #upload directory
):

    url = "https://api.pinata.cloud/pinning/pinJSONToIPFS"

    payload = json.dumps({
      "pinataOptions": {
        "cidVersion": cid_version
      },
      "pinataMetadata": {
        "name": name,
        "keyvalues": metadata
      },
      "pinataContent": {"file":fpaths}
    })
    headers = {
        'Authorization': f'Bearer {cred}',
        'Content-Type': 'application/json'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    return response

# %% ../nbs/03_pinataapi.ipynb 21
def pin(cred:str,#JWT key
        cid:str, #IPFS cid
        fn=None, #Name of file
        pinataMetadata=None #Add keys and values associated with IPFS CID
):

    url = "https://api.pinata.cloud/pinning/pinByHash"

    payload = json.dumps({
      "hashToPin": cid,
      "pinataMetadata": {
        "name": fn,
        "keyvalues": pinataMetadata
      }
    })
    headers = {
      'Authorization': f'Bearer {cred}',
      'Content-Type': 'application/json'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print(response.text)


# %% ../nbs/03_pinataapi.ipynb 24
def unpin(cred:str,#JWT Key
          cid:str #IPFS CID
):
    
    url = f"https://api.pinata.cloud/pinning/unpin/{cid}"

    payload={}
    headers = {
      'Authorization': f'Bearer {cred}'
    }

    response = requests.request("DELETE", url, headers=headers, data=payload)

    print(response)

# %% ../nbs/03_pinataapi.ipynb 29
def edit_metadata(cred:str,#JWT Key
              cid:str, #IPFS CID
              name:str, #filename
              metadata=None #Add keys and values associated with IPFS CID
):

    url = "https://api.pinata.cloud/pinning/hashMetadata"
    
    pinataMetadata = dict({"name":name,"keyvalues":{}})
    pinataMetadata["keyvalues"].update(metadata)
    pinataMetadata["ipfsPinHash"] = cid

    payload = json.dumps(pinataMetadata)
    headers = {
      'Authorization': f'Bearer {cred}',
      'Content-Type': 'application/json'
    }

    response = requests.request("PUT", url, headers=headers, data=payload)

    return response


# %% ../nbs/03_pinataapi.ipynb 34
def get_pinned_jobs(cred:str,# JWT Key
                    params=None # filtering pinned jobs
):

    '''
    'sort' - Sort the results by the date added to the pinning queue (see value options below)
    'ASC' - Sort by ascending dates
    'DESC' - Sort by descending dates
    'status' - Filter by the status of the job in the pinning queue (see potential statuses below)
    'prechecking' - Pinata is running preliminary validations on your pin request.
    'searching' - Pinata is actively searching for your content on the IPFS network. This may take some time if your content is isolated.
    'retrieving' - Pinata has located your content and is now in the process of retrieving it.
    'expired' - Pinata wasn't able to find your content after a day of searching the IPFS network. Please make sure your content is hosted on the IPFS network before trying to pin again.
    'over_free_limit' - Pinning this object would put you over the free tier limit. Please add a credit card to continue pinning content.
    'over_max_size' - This object is too large of an item to pin. If you're seeing this, please contact us for a more custom solution.
    'invalid_object' - The object you're attempting to pin isn't readable by IPFS nodes. Please contact us if you receive this, as we'd like to better understand what you're attempting to pin.
    'bad_host_node' - You provided a host node that was either invalid or unreachable. Please make sure all provided host nodes are online and reachable.
    'ipfs_pin_hash' - Retrieve the record for a specific IPFS hash
    'limit' - Limit the amount of results returned per page of results (default is 5, and max is 1000)
    'offset' - Provide the record offset for records being returned. This is how you retrieve records on additional pages (default is 0)
    '''

    base_url = 'https://api.pinata.cloud/pinning/pinJobs/'

    header = {'Authorization': f'Bearer {cred}'}

    response = requests.get(base_url, headers=header,params=params)

    return response

# %% ../nbs/03_pinataapi.ipynb 37
def get_pinned_files(cred:str,# JWT Key
                     params=None # Filter returned pinned files 
):

    '''
    Query Parameters = params

    hashContains: (string) - Filter on alphanumeric characters inside of pin hashes. Hashes which do not include the characters passed in will not be returned.
    pinStart: (must be in ISO_8601 format) - Exclude pin records that were pinned before the passed in 'pinStart' datetime.
    pinEnd: (must be in ISO_8601 format) - Exclude pin records that were pinned after the passed in 'pinEnd' datetime.
    unpinStart: (must be in ISO_8601 format) - Exclude pin records that were unpinned before the passed in 'unpinStart' datetime.
    unpinEnd: (must be in ISO_8601 format) - Exclude pin records that were unpinned after the passed in 'unpinEnd' datetime.
    pinSizeMin: (integer) - The minimum byte size that pin record you're looking for can have
    pinSizeMax: (integer) - The maximum byte size that pin record you're looking for can have
    status: (string) -
        * Pass in 'all' for both pinned and unpinned records
        * Pass in 'pinned' for just pinned records (hashes that are currently pinned)
        * Pass in 'unpinned' for just unpinned records (previous hashes that are no longer being pinned on pinata)
    pageLimit: (integer) - This sets the amount of records that will be returned per API response. (Max 1000)
    pageOffset: (integer) - This tells the API how far to offset the record responses. For example,
    if there's 30 records that match your query, and you passed in a pageLimit of 10, providing a pageOffset of 10 would return records 11-20.
    '''

    base_url = 'https://api.pinata.cloud/data/pinList?'

    header = {'Authorization': f'Bearer {cred}'}

    response = requests.get(base_url, headers=header,params=params)

    return response

# %% ../nbs/03_pinataapi.ipynb 40
def get_datausage(cred:str,# JWT Key
                  params=None # Filter returned data usage statistics 
):

    header = {'Authorization': f'Bearer {cred}'}

    base_url = 'https://api.pinata.cloud/data/userPinnedDataTotal'

    response = requests.get(base_url, headers=header,params=params)

    return response
